/*
 * Kerio IP filter
 * Copyright (C) Kerio Technologies s.r.o.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "hdrs.h"

//
//	DETAILED_QUEUE_USAGE_LOG
//
//#define DETAILED_QUEUE_USAGE_LOG

void
queue_enable(
	struct ipf_queue *queue,
	int enable
	)
{
	spin_lock_bh(&queue->lock);
	queue->enabled = enable;
	spin_unlock_bh(&queue->lock);
}

int
queue_cmp_by_id(
	struct sk_buff *skb,
	void *id
	)
{
	return ((skb == id) ? (0) : (1));
}

int
queue_cmp_by_dev(
	struct sk_buff *skb,
	void *dev
	)
{
	return (skb && skb->dev == dev) ? 0 : 1;
}

int
queue_len(
	struct ipf_queue *queue
	)
{
	int len = 0;

	spin_lock_bh(&queue->lock);

	len = queue->queued_pkts;

	spin_unlock_bh(&queue->lock);

	return len;
}

int
queue_is_empty(
	struct ipf_queue *queue
	)
{
	int ret = 0;

	spin_lock_bh(&queue->lock);

	ret = ((queue->queued_pkts) ? (0) : (1));

	spin_unlock_bh(&queue->lock);

	return ret;
}

int
queue_insert(
	struct ipf_queue *queue,
	struct sk_buff *skb,
	int tail
	)
{
	spin_lock_bh(&queue->lock);

	if (!queue->enabled) {
		spin_unlock_bh(&queue->lock);
		return -EINVAL;
	}

	if (tail) {
		__skb_queue_tail(&queue->q, skb);
	} else {
		__skb_queue_head(&queue->q, skb);
	}

	queue->queued_pkts++;

#if defined (DETAILED_QUEUE_USAGE_LOG)
	TRACE(LOG_DBG,"%s: skb=%p queued.", __func__, skb);
#endif

	spin_unlock_bh(&queue->lock);

	return 0;
}

struct sk_buff*
queue_remove(
	struct ipf_queue *queue,
	void *data,
	ipf_cmpfn cmpfn
	)
{
	struct sk_buff *skb = NULL;

	spin_lock_bh(&queue->lock);

	skb = ((struct sk_buff *)&queue->q)->next;

	while (skb != (struct sk_buff *)&queue->q) {

		if (!cmpfn || cmpfn(skb, data) == 0) {

			__skb_unlink(skb, &queue->q);

			queue->queued_pkts--;

			spin_unlock_bh(&queue->lock);

#if defined (DETAILED_QUEUE_USAGE_LOG)
			TRACE(LOG_DBG,"%s: skb=%p removed from queue.", __func__, skb);
#endif
			return skb;
		}

		skb = skb->next;
	}

	spin_unlock_bh(&queue->lock);

	return NULL;
}

void
queue_init(
	struct ipf_queue *queue
	)
{
	//
	//	ToDo: add initialized variable
	//
	skb_queue_head_init(&queue->q);

	alloc_lock(&queue->lock);

	queue->queued_pkts = 0;
	queue->enabled = 0;
}

void
queue_close(
	struct ipf_queue *queue
	)
{
	int len = queue_len(queue);

	if (len) {
		TRACE(LOG_WARN, "%s: ### %d buffers still in queue. ###", __func__, len);
	}

	free_lock(&queue->lock);
}
